import { APIClient } from "@cross-lab-project/api-client";
import { DeviceHandler } from "@cross-lab-project/soa-client";
import { adoptStyles, html, LitElement, unsafeCSS } from "lit";
import { customElement, state } from "lit/decorators.js";
import { ElectricalConnection } from "./electricalConnection";
import { FileUpload } from './file';
import { Webcam } from "./webcam";

let device_url = "";
const client = new APIClient("https://api.goldi-labs.de");
const deviceHandler = new DeviceHandler();

import style from "./styles.css";
import { Message } from "./message";
const stylesheet = unsafeCSS(style);
@customElement("ecp-app")
export class App extends LitElement {
  @state()
  private electrical: ElectricalConnection;
  @state()
  private file: FileUpload;
  @state()
  private webcam: Webcam;
  @state()
  private message: Message;

  @state()
  private isLoading: boolean = false;

  async connectedCallback() {
    super.connectedCallback();
    adoptStyles(this.shadowRoot, [stylesheet]);

    window.addEventListener("message", (event) => {
      if(event.data.token){
        device_url = event.data.device_url;
        this.start(event.data.token)
      }
    });
    // Send a message to the parent window
    window.parent.postMessage("ecp-loaded", "*");
    window.opener.postMessage("ecp-loaded", "*")
  }

  async start(accesstoken: string) {
    console.log({accesstoken, device_url})
    client.accessToken=accesstoken;
    const token = await client.createWebsocketToken(device_url);

    this.electrical = new ElectricalConnection("electrical");
    this.electrical.register(deviceHandler);

    this.file = new FileUpload("file");
    this.file.register(deviceHandler);

    this.webcam = new Webcam();
    this.webcam.register(deviceHandler);

    this.message = new Message("message");
    this.message.register(deviceHandler);

    await deviceHandler.connect({
      endpoint: "wss://api.goldi-labs.de/devices/websocket",
      id: device_url,
      token,
    });

    // Wait for a second
    await new Promise((resolve) => setTimeout(resolve, 1000));
    window.parent.postMessage("ecp-authorized", "*");
    window.opener.postMessage("ecp-authorized", "*");
  }

  render() {
    return html`
      <div class="h-full flex flex-col" @webcam="${() => {this.isLoading = false;}}">
        <div class="bg-primary-900 w-full h-12"></div>
        <div class="grow flex w-full items-center">
          <div class="flex-1">${this.webcam}</div>
          <div class="flex-1 flex flex-col">
            <div class="p-4">${this.file}</div>
            <div class="p-4">${this.electrical}</div>
          </div>
        </div>
        <div>
          ${this.message}
        </div>
        <div class="bg-primary-100 w-full h-12"></div>
      </div>
      ${
        this.isLoading
          ? html`<div
              class="absolute top-0 left-0 w-full h-full bg-primary-900 bg-opacity-50 flex items-center justify-center"
            >
              Loading...
            </div>`
          : html``
      }
    `;
  }
}